(function() {
    "use strict";

    angular.module("weChatApp", ["ngRoute", "ngAnimate"])
        .config(["$routeProvider", AppConfig])
    ;

    function AppConfig($routeProvider) {
        $routeProvider
            .when("/", {
                templateUrl: "views/wechat-rep-dashboard.html",
                controller: "dashboardCtrl",
                controllerAs: "dash",
                resolve: {
                    _currentUser: ["$q", "UserService", function($q, userService) {
                        var deferred = $q.defer();

                        userService.getCurrentUser().then(function(currentUser) {
                            deferred.resolve(currentUser);
                        });

                        return deferred.promise;
                    }],
                    _veevaMessages: ["$q", "UserService", "TranslationService", function($q, userService, translationService) {
                        var deferred = $q.defer();

                        userService.getCurrentUser().then(function(currentUser) {
                            translationService.getVeevaMessages(currentUser.LanguageLocaleKey, "en_US").then(function(resp) {
                                deferred.resolve(resp);
                            });
                        });

                        return deferred.promise;
                    }],
                    _unreachableObjects: ["$q", "Queries", "PermissionService", function($q, queries, permissionService) {
                        var deferred = $q.defer();

                        permissionService.checkObjectAndFieldFLS(queries).then(function(unreachableObjects) {
                            /**
                             * The unreachableObjects can be used to check missing FLS on object, fields. Its structure is
                             * { object_api_name_1: [], object_api_name_2: [] }
                             *
                             * For object missing field(s) access, the unreachableObjects structure will be
                             * { object_api_name_1: [field1], object_api_name_2: [field1, field2] }
                             */
                            deferred.resolve(unreachableObjects);
                        });

                        return deferred.promise;
                    }]
                }
            })
        ;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .controller("dashboardCtrl", ["$scope", "$timeout", "$q", "_currentUser", "_unreachableObjects", "DashboardHelper",
            "TranslationService", "SentMessageService", "TerritoryService", "ConsentService", "TsfService", "Utility",
            "ChartJsPlugins", "AppConstants", "Queries", DashboardCtrl])
    ;

    function DashboardCtrl($scope, $timeout, $q, _currentUser, _unreachableObjects, dashboardHelper,
                           translationService, sentMessageService, territoryService, consentService, tsfService, utility,
                           chartJsPlugins, appConstants, queries) {
        var dash = this;
        var OBJECT_LABELS = {};
        var FIELD_LABELS = {};
        var PICKLIST_LABELS = {};
        var user_timezone;
        var user_locale;

        dashboardHelper.excludeUnreachableFieldFromQueries(queries, _unreachableObjects);
        dashboardHelper.getLabels(_unreachableObjects).then(function(resp) {
            OBJECT_LABELS = resp.objectLabels;
            FIELD_LABELS = resp.fieldLabels;
            PICKLIST_LABELS = resp.picklistLabels;
            preStartSetup();
            start();
        });

        function preStartSetup() {
            user_timezone = _currentUser.TimeZoneSidKey;
            user_locale = _currentUser.LocaleSidKey;

            if(!moment.tz.zone(user_timezone)) {
                user_timezone = "GMT";
            }

            moment.locale("en");
            if(user_locale) {
                moment.locale(user_locale); // moment will ignore the locale here if it's not valid
            }

            dash.detailMode = false;
        }

        function start() {
            initGlobalDateFilter();

            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(getFilterDates());
            }
            if(dashboardHelper.okToEnableTile(appConstants.CONSENT, _unreachableObjects)) {
                getConsents();
            }
        }

        function initGlobalDateFilter() {
            dash.globalDateFilterOptions = [];
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 30]),
                dayRange: -30
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 60]),
                dayRange: -60
            });
            dash.globalDateFilterOptions.push({
                label: translationService.getMessage.apply(null, ["LAST_DAYS", 90]),
                dayRange: -90
            });
            dash.selectedDateFilter = dash.globalDateFilterOptions[0];
        }

        function getFilterDates() {
            var backward = dash.selectedDateFilter.dayRange !== Math.abs(dash.selectedDateFilter.dayRange);
            var dayRange = Math.abs(dash.selectedDateFilter.dayRange);

            var filterEndDate;
            var filterStartDate;
            if(backward) {
                filterEndDate = moment(); // now
                filterStartDate = filterEndDate.clone().subtract(dayRange, "days");
            } else {
                filterStartDate = moment(); // now
                filterEndDate = filterStartDate.clone().add(dayRange, "days");
            }

            var startDateUTC = moment.utc(filterStartDate);
            var endDateUTC = moment.utc(filterEndDate);
            return {
                startDatetime: startDateUTC.toISOString(),
                endDatetime: endDateUTC.toISOString(),
                startDate: startDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                endDate: endDateUTC.format(appConstants.YEAR_MONTH_DAY_FORMAT),
                startDateMoment: startDateUTC,
                endDateMoment: endDateUTC
            };
        }

        function getSentMessages(filterDates) {
            var includeAccountName = true;
            var includeMultichannelContent = true;
            var includeContentType = true;
            if(!!_unreachableObjects.Sent_Message_vod__c) {
                includeAccountName = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Account"));
                includeMultichannelContent = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Multichannel_Content_vod__c"));
                includeContentType = !~(_unreachableObjects.Sent_Message_vod__c.indexOf("Content_Type_vod__c"));
            }

            sentMessageService.getSentMessageByUserId(_currentUser.Id, filterDates, includeAccountName,
                includeMultichannelContent, includeContentType).then(function(sentMessageResp) {
                console.info(sentMessageResp);

                var openedSMs = sentMessageResp.filter(function(sm) {
                    // for WM discrepancy
                    if(typeof sm.Opened_vod__c === "string") {
                        return !!parseInt(sm.Opened_vod__c);
                    }
                    return !!sm.Opened_vod__c;
                });

                dash.sentMessageResp = sentMessageResp;
                dash.messageOpenRate = 0;

                if(sentMessageResp.length > 0) {
                    dash.messageOpenRate = (100 * openedSMs.length / sentMessageResp.length).toFixed();
                }
                dash.sentMessageReady = true;
            });
        }

        function getConsents() {
            dash.hcpOptInPercentage = 0;
            dash.hcpSentMessageCoverPercentage = 0;

            // Get accounts under user territory
            territoryService.getAccountsUnderUserTerritory(_currentUser.Id).then(function(accountResp) {
                // Prepare a list of TSF external ids to use later
                // TSF external id is formatted as "AccountID__TerritoryName"
                var tsfExtIds = [];
                accountResp.forEach(function(acct) {
                    acct.territories.forEach(function(acctTerritory) {
                        tsfExtIds.push(acct.Id + "__" + acctTerritory.Name);
                    });
                });

                // Get target accounts from TSF table using TSF external ids
                var targetAccounts = [];
                var targetAccountTracker = {};
                tsfService.getTargetAccountByExternalIds(tsfExtIds).then(function(tsfResp) {
                    tsfResp.forEach(function(tsf) {
                        if(!targetAccountTracker[tsf.Account_vod__c]) {
                            targetAccounts.push(_.find(accountResp, function(acct) {
                                return acct.Id === tsf.Account_vod__c;
                            }));
                            targetAccountTracker[tsf.Account_vod__c] = true;
                        }
                    });
                    console.info(targetAccounts);

                    // Get WeChat Consent by targeted accounts
                    /** Include content type query if
                     *  1. has OLS on Content_Type_vod__c (i.e. _unreachableObjects.Content_Type_vod__c is undefined)
                     *  2. has FLS on Content_Type_vod__c.Name (i.e. _unreachableObjects.Content_Type_vod__c does not contain "Name")
                     */
                    var includeContentType = true;
                    if(!!_unreachableObjects.Content_Type_vod__c) {
                        includeContentType = _unreachableObjects.Content_Type_vod__c.length !== 0 && !~(_unreachableObjects.Content_Type_vod__c.indexOf("Name"));
                    }
                    consentService.getWeChatConsentByAccounts(targetAccounts, includeContentType).then(function(consentResp) {
                        console.info(consentResp);
                        dash.consentResp = consentResp;

                        /** To be considered as opt-in consent, the record need to
                         *  1. Opt_Type_vod__c = Opt_In_vod
                         *  2. Opt_Expiration_Date_vod__c >= TODAY OR EMPTY
                         */
                        var activeOptInConsent = consentResp.filter(function(c) {
                            return c.Opt_Type_vod__c === "Opt_In_vod" && (!c.Opt_Expiration_Date_vod__c ||
                                utility.parseStringToDateValue(c.Opt_Expiration_Date_vod__c) >= moment.utc().valueOf());
                        });
                        var targetAccountsWithActiveOptInConsent = _.uniqBy(activeOptInConsent, "Account_vod__c");

                        if(targetAccounts.length > 0) {
                            dash.hcpOptInPercentage = (100 * targetAccountsWithActiveOptInConsent.length / targetAccounts.length).toFixed();
                        }

                        $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                            name: "OPT_IN_RATE",
                            data: dash.hcpOptInPercentage
                        });
                    });

                    // Get Sent Messages for targeted user
                    sentMessageService.getSentMessageByUserId(_currentUser.Id).then(function(allSentMessages) {
                        var targetAccountsWithSentMessage = _.uniqBy(allSentMessages, "Account_vod__c").filter(function(sm) {
                            // target accounts must match account from the right territory
                            return !!_.find(targetAccounts, function(account) {
                                return account.Id === sm.Account_vod__c;
                            });
                        });

                        if(targetAccounts.length > 0) {
                            dash.hcpSentMessageCoverPercentage = (100 * targetAccountsWithSentMessage.length / targetAccounts.length).toFixed();
                        }

                        $scope.$broadcast(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, {
                            name: "SENT_MESSAGE_COVERAGE",
                            data: dash.hcpSentMessageCoverPercentage
                        });
                    });

                    dash.consentReady = true;
                });
            });
        }

        dash.globalFilterOnChange = function() {
            if(dashboardHelper.okToEnableTile(appConstants.SENT_MESSAGE, _unreachableObjects)) {
                getSentMessages(getFilterDates());
            }
        };

        dash.toggleDetailMode = function() {
            dash.detailMode = !dash.detailMode;
            if(!dash.detailMode) {
                dash.showSentMessageDetail = false;
                dash.showConsentDetail = false;
            }
        };

        dash.viewAllSentMessages = function() {
            var fieldLabels = FIELD_LABELS.Sent_Message_vod__c;
            dash.sentMessageTableHeaders = [{
                label: fieldLabels.Content_vod__c,
                name: "Content_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Sent_Datetime_vod__c,
                name: "Sent_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }, {
                label: translationService.getMessage('OPENED'),
                name: "Status",
                isDate: false
            }, {
                label: fieldLabels.Last_Open_Datetime_vod__c,
                name: "Last_Open_Datetime_vod__c",
                isDate: true
            }, {
                label: fieldLabels.Total_Duration_vod__c,
                name: "Total_Duration_vod__c",
                isDate: false
            }, {
                label: translationService.getMessage('SERVICE_ACCOUNT'),
                name: "Content_Type_vod__c",
                isDate: false
            }];

            dash.sentMessageTableBody = [];
            dash.sentMessageInfoCard = [];
            dash.sentMessageResp.forEach(function(sm) {
                dash.sentMessageTableBody.push({
                    Content_vod__c: sm.contentName,
                    Account_vod__c: sm.accountName,
                    Sent_Datetime_vod__c: utility.parseStringToDateValue(sm.Sent_Datetime_vod__c),
                    Status: sm.Last_Open_Datetime_vod__c ? translationService.getMessage("OPENED") : translationService.getMessage("UNOPENED"),
                    Last_Open_Datetime_vod__c: utility.parseStringToDateValue(sm.Last_Open_Datetime_vod__c),
                    Total_Duration_vod__c: sm.Total_Duration_vod__c,
                    Content_Type_vod__c: sm.serviceAccountName
                });
                // fill up the info card for detail view (iPhone)
                dash.sentMessageInfoCard.push(_.assign({
                    cardHeader: OBJECT_LABELS.Sent_Message_vod__c.singular + " " + dash.sentMessageTableBody.length
                }, _.last(dash.sentMessageTableBody)));
            });

            // sort the info card in descending order by Sent Datetime
            dash.sentMessageInfoCard.sort(function(sm1, sm2) {
                if(sm1.Sent_Datetime_vod__c && sm2.Sent_Datetime_vod__c) {
                    return sm2.Sent_Datetime_vod__c - sm1.Sent_Datetime_vod__c;
                } else if (sm1.Sent_Datetime_vod__c) {
                    return -1;
                } else if (sm2.Sent_Datetime_vod__c) {
                    return 1;
                } else {
                    return 0;
                }
            });

            dash.sentMessageObjectLabel = OBJECT_LABELS.Sent_Message_vod__c.plural;
            dash.showSentMessageDetail = true;
            dash.toggleDetailMode();
        };

        dash.viewAllConsents = function() {
            var fieldLabels = FIELD_LABELS.Multichannel_Consent_vod__c;
            dash.consentTableHeaders = [{
                label: fieldLabels.Account_vod__c,
                name: "Account_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Opt_Type_vod__c,
                name: "Opt_Type_vod__c",
                isDate: false
            }, {
                label: fieldLabels.Capture_Datetime_vod__c,
                name: "Capture_Datetime_vod__c",
                isDate: true,
                defaultSort: true
            }, {
                label: translationService.getMessage('SERVICE_ACCOUNT'),
                name: "ServiceAccount",
                isDate: false
            }, {
                label: fieldLabels.Opt_Expiration_Date_vod__c,
                name: "Opt_Expiration_Date_vod__c",
                isDate: true
            }];
            dash.consentTableBody = [];
            dash.consentInfoCard = [];
            dash.consentResp.forEach(function(c) {
                dash.consentTableBody.push({
                    Account_vod__c: c.accountName,
                    Opt_Type_vod__c: PICKLIST_LABELS.Multichannel_Consent_vod__c.Opt_Type_vod__c[c.Opt_Type_vod__c],
                    Capture_Datetime_vod__c: utility.parseStringToDateValue(c.Capture_Datetime_vod__c),
                    ServiceAccount: c.contentName,
                    Opt_Expiration_Date_vod__c: utility.parseStringToDateValue(c.Opt_Expiration_Date_vod__c)
                });
            });

            // we only contain the 50 latest consent by capture datetime for the detail view
            dash.consentTableBody.sort(function(c1, c2) {
                if(c1.Capture_Datetime_vod__c && c2.Capture_Datetime_vod__c) {
                    return c2.Capture_Datetime_vod__c - c1.Capture_Datetime_vod__c;
                } else if (c1.Capture_Datetime_vod__c) {
                    return -1;
                } else if (c2.Capture_Datetime_vod__c) {
                    return 1;
                } else {
                    return 0;
                }
            });
            dash.consentTableBody.splice(50);

            // fill up the info card for detail view (iPhone)
            dash.consentTableBody.forEach(function(c, i) {
                dash.consentInfoCard.push(_.assign({
                    cardHeader: OBJECT_LABELS.Multichannel_Consent_vod__c.singular + " " + (++i)
                }, c));
            });

            dash.consentObjectLabel = OBJECT_LABELS.Multichannel_Consent_vod__c.plural;
            dash.showConsentDetail = true;
            dash.toggleDetailMode();
        };

        // register ChartJS custom plug-ins
        Chart.pluginService.register(chartJsPlugins.centerLabel);
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .directive("customTable", Table)
    ;

    function Table() {
        return {
            restrict: "E",
            templateUrl: "views/custom-table.html",
            scope: {
                headers: '<',
                rows: '<'
            },
            controller: ["$scope", function($scope) {
                var defaultSortingHeader = _.find($scope.headers, function(header) {
                    return header.defaultSort;
                });
                if(defaultSortingHeader) {
                    $scope.orderKey = "-" + defaultSortingHeader.name;
                } else if($scope.headers[0]) {
                    $scope.orderKey = "-" + $scope.headers[0].name;
                }

                $scope.sortColumn = function(header) {
                    if($scope.orderKey === header.name) {
                        $scope.orderKey = "-" + header.name;
                    } else {
                        $scope.orderKey = header.name;
                    }
                };
            }]
        };
    }
}());
(function() {
    'use strict';

    angular.module('weChatApp')
        .directive('dashboardTile', DashboardTile)
    ;

    function DashboardTile() {
        return {
            restrict: 'E',
            transclude: true,
            templateUrl: 'views/dashboard-tile.html',
            scope: {
                show: '<',
                mainTitle: '@',
                subTitle: '<',
                extraTitleInfoTile: '<',
                statusList: '<',
                actionLabel: '@',
                actionCallback: '&'
            }
        };
    }
}());
(function() {
    'use strict';

    angular.module('weChatApp')
        .directive('doughnutChart', ['AppConstants', DoughnutChart])
    ;

    function DoughnutChart(appConstants) {
        var ONE_HUNDRED = 100;

        return {
            template: '<canvas></canvas>',
            restrict: 'E',
            replace: true,
            transclude: true,
            scope: {
                name: '@',
                width: '@',
                height: '@',
                color1: '@',
                color2: '@',
                percentage: '='
            },
            link: function(scope, element) {
                if(scope.width) {
                    element.css('width', scope.width);
                }
                if(scope.height) {
                    element.css('height', scope.height);
                }

                var gradient = element[0].getContext("2d").createLinearGradient(0, 0, 0, 200);
                gradient.addColorStop(0, scope.color1);
                gradient.addColorStop(1, scope.color2);

                var adjustedPercentage = Math.min(parseInt(scope.percentage), ONE_HUNDRED);
                var data = {
                    datasets: [{
                        data: [adjustedPercentage, ONE_HUNDRED - adjustedPercentage],
                        backgroundColor: [
                            gradient,
                            "rgb(205,205,205)"
                        ]
                    }]
                };

                var options = {
                    cutoutPercentage: 80,
                    responsive: false,
                    events: [],
                    customLabel: 'label',
                    customLabelColor: '#666666',
                    centerLabel: {
                        text1: adjustedPercentage + '%',
                        text1Color: 'rgb(102,102,102)'
                    } // enable center label plugin
                };

                var doughnutChart = new Chart(element, {
                    type: 'doughnut',
                    data: data,
                    options: options
                });

                scope.$on(appConstants.UPDATE_DOUGHNUT_CHART_EVENT, function(event, params) {
                    if(params.name === scope.name) {
                        adjustedPercentage = Math.min(parseInt(params.data), ONE_HUNDRED);
                        doughnutChart.data.datasets[0].data = [adjustedPercentage, ONE_HUNDRED - adjustedPercentage];
                        doughnutChart.options.centerLabel.text1 = adjustedPercentage + '%';
                        doughnutChart.update();
                    }
                });
            }
        };
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .directive("infoCard", InfoCard)
    ;

    function InfoCard() {
        return {
            restrict: "E",
            templateUrl: "views/info-card.html",
            scope: {
                mainTitle: '<',
                subTitle: '<',
                cardHeaders: '<',
                cardBody: '<'
            }
        };
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .filter("DateFormatter", [DateFormatter])
    ;

    function DateFormatter() {
        return function(input, enable) {
            if(enable && input) {
                return moment(input).format("LLL");
            } else {
                return input;
            }
        };
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .filter("PercentageFormatter", [PercentageFormatter])
    ;

    function PercentageFormatter() {
        return function(input) {
            if(typeof input === "number" || typeof input === "string") {
                return input + " %";
            } else {
                return "";
            }
        };
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .filter("TranslationFilter", ["TranslationService", TranslationFilter])
    ;

    function TranslationFilter(translationService) {
        return function(input) {
            if(angular.isArray(input)) {
                return translationService.getMessage.apply(null, input);
            } else {
                return translationService.getMessage(input);
            }
        };
    }
}());
(function() {
  'use strict';

  angular.module('weChatApp').factory('ChartJsPlugins', [factoryFn]);

  function factoryFn() {
    var plugins = this;

    function isPluginEnabled(chartInstance, pluginName) {
      return chartInstance.options[pluginName] && (typeof chartInstance.options[pluginName] === 'object' || typeof chartInstance.options[pluginName] === 'boolean');
    }

    plugins.centerLabel = {
      pluginName: 'centerLabel',

      beforeDraw: function(chart) {
        if(!isPluginEnabled(chart, this.pluginName)) {
          return;
        }

        if(chart.config.type === 'doughnut') {
          var width = chart.chart.width;
          var height = chart.chart.height;
          var ctx = chart.chart.ctx;

          ctx.restore();

          var fontSize = 1;
          ctx.font = "normal normal 100 "+ fontSize + "em 'Lato-Light'";
          ctx.textBaseline = "middle";

          if(chart.config.options[this.pluginName].text1Color) {
            ctx.fillStyle = chart.config.options[this.pluginName].text1Color;
          }

          var text = chart.config.options[this.pluginName].text1;
          var textX = Math.round((width - ctx.measureText(text).width) / 2);
          var textY = fontSize * 2 + height * 0.5;

          ctx.fillText(text, textX, textY);
          ctx.save();
        }
      }
    };

    return plugins;
  }
})();

(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("ConsentService", ["$q", "Endpoints", ConsentService])
    ;

    function ConsentService($q, endpoints) {
        var service = this;

        service.getWeChatConsentByAccounts = function(accounts, includeContentType) {
            var deferred = $q.defer();
            var accountIds = [];
            var accountIdToNameMap = {};

            accounts.forEach(function(account) {
                accountIds.push(account.Id);
                accountIdToNameMap[account.Id] = account.Name;
            });

            endpoints.fetchMultichannelConsentByAccountIds(accountIds).then(function(rawResp) {
                var consentResp = rawResp.Multichannel_Consent_vod__c;

                // fetch RecordType id for wechat_vod
                endpoints.fetchWeChatRecordType().then(function(recordTypeResp) {
                    var weChatRecordTypeId = recordTypeResp.RecordType[0] ? recordTypeResp.RecordType[0].Id : null;

                    // filter mc consent by record type id to get WeChat consent only
                    var weChatConsentResp = consentResp.filter(function(consentData) {
                        return consentData.RecordTypeId === weChatRecordTypeId;
                    });

                    weChatConsentResp.forEach(function(consent) {
                        if(!!consent.Account_vod__c) {
                            consent.accountName = accountIdToNameMap[consent.Account_vod__c];
                        }
                    });

                    // Look up content type name and attach to the response
                    if(includeContentType) {
                        var contentTypeIds = _.map(_.uniqBy(weChatConsentResp, "Content_Type_vod__c"), "Content_Type_vod__c");
                        endpoints.fetchContentTypeByIds(contentTypeIds).then(function(contentResp) {
                            var contentTypeIdToNameMap = generateIdToNameMap(contentResp.Content_Type_vod__c);
                            weChatConsentResp.map(function(consent) {
                                if(!!consent.Content_Type_vod__c) {
                                    consent.contentName = contentTypeIdToNameMap[consent.Content_Type_vod__c] || '';
                                }
                                return consent;
                            });
                            deferred.resolve(weChatConsentResp);
                        });
                    } else {
                        deferred.resolve(weChatConsentResp);
                    }

                }, deferred.reject);
            }, deferred.reject);
            return deferred.promise;
        };

        function generateIdToNameMap(objectList) {
            var map = {};
            objectList.forEach(function(obj) {
                map[obj.Id] = obj.Name;
            });
            return map;
        }

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("DashboardHelper", ["$q", "AppConstants", "TranslationService", "FieldLabels", "PicklistLabels",
            "RecordTypeLabels", MedicalDashboardHelper])
    ;

    function MedicalDashboardHelper($q, appConstants, translationService, fieldLabels, picklistLabels,
                                    recordTypeLabels) {
        var helper = this;

        helper.excludeUnreachableFieldFromQueries = function(queries, unreachableObjects) {
            for(var key in queries) {
                if(Object.prototype.hasOwnProperty.call(queries, key)) {
                    var objectName = queries[key].object;
                    var queryFields = queries[key].fields;
                    var unreachableFields = unreachableObjects[objectName];
                    if(!!unreachableFields && unreachableFields.length > 0) {
                        // exclude query field(s) that is not accessible
                        console.warn('Excluding query field(s): ' + unreachableFields.join(',') + ' from ' + objectName);
                        queries[key].fields = _.difference(queryFields, unreachableFields);
                    }
                }
            }
        };

        helper.getLabels = function(unreachableObjects) {
            var deferred = $q.defer();
            var objectLabelDeferred = $q.defer();
            var fieldLabelDeferred = $q.defer();
            var picklistLabelDeferred = $q.defer();
            var recordTypeLabelDeferred = $q.defer();
            var promises = [
                objectLabelDeferred.promise,
                fieldLabelDeferred.promise,
                picklistLabelDeferred.promise,
                recordTypeLabelDeferred.promise
            ];

            var fieldLabelToTranslate = angular.copy(fieldLabels);
            var picklistLabelToTranslate = angular.copy(picklistLabels);
            var recordTypeLabelToTranslate = _.differenceBy(recordTypeLabels, _.keys(unreachableObjects));
            for(var key in unreachableObjects) {
                if(Object.prototype.hasOwnProperty.call(unreachableObjects, key)) {
                    var unreachableFields = unreachableObjects[key];
                    if(unreachableFields.length > 0) {
                        fieldLabelToTranslate[key] = _.difference(fieldLabelToTranslate[key], unreachableFields);
                    } else {
                        delete fieldLabelToTranslate[key];
                        delete picklistLabelToTranslate[key];
                    }
                }
            }

            translationService.getObjectLabels(fieldLabelToTranslate).then(function(resp) {
                objectLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(objectLabelDeferred));
            translationService.getFieldLabels(fieldLabelToTranslate).then(function(resp) {
                fieldLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(fieldLabelDeferred));
            translationService.getPicklistLabels(picklistLabelToTranslate).then(function(resp) {
                picklistLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(picklistLabelDeferred));
            translationService.getRecordTypeLabels(recordTypeLabelToTranslate).then(function(resp) {
                recordTypeLabelDeferred.resolve(resp);
            }, genericQueryErrorHandler(recordTypeLabelDeferred));

            $q.all(promises).then(function(comboResp) {
                deferred.resolve({
                    objectLabels: comboResp[0],
                    fieldLabels: comboResp[1],
                    picklistLabels: comboResp[2],
                    recordTypeLabels: comboResp[3]
                });
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        helper.okToEnableTile = function(tileName, unreachableObjects) {
            var enable = true;

            if(tileName === appConstants.SENT_MESSAGE) {
                enable = !angular.isDefined(unreachableObjects.Sent_Message_vod__c) || unreachableObjects.Sent_Message_vod__c.length > 0;
            }
            else if(tileName === appConstants.CONSENT) {
                enable = (!angular.isDefined(unreachableObjects.Multichannel_Consent_vod__c) || unreachableObjects.Multichannel_Consent_vod__c.length > 0)
                    && !angular.isDefined(unreachableObjects.Territory)
                    && !angular.isDefined(unreachableObjects.UserTerritory);
            }

            if(!enable) {
                console.warn(tileName + " is not enabled");
            }
            return enable;
        };

        function genericQueryErrorHandler(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        }

        return helper;
    }
}());

(function() {
    'use strict';

    angular.module('weChatApp')
        .factory('Endpoints', ["$q", "Queries", Endpoints])
    ;

    function Endpoints($q, queries) {
        var endpoints = this;
        var utils = new window.VeevaUtilities();

        var normalizeIdForOnline = function(responseObject) {
            if(angular.isObject(responseObject) && angular.isDefined(responseObject.ID)) {
                responseObject.Id = responseObject.ID;
            }
        };

        var normalizeWhereClauseBooleanValue = function(value) {
            if(utils.isWin8() && !utils.isOnline()) {
                return (typeof value === 'boolean' && value) ? '\'true\'' : '\'false\'';
            }
            return typeof value === 'boolean' && value;
        };

        var normalizeWhereClauseDateValue = function(value) {
            if(utils.isOnline()) {
                return value;
            } else {
                return "'" + value + "'";
            }
        };

        var normalizeWhereClauseEmptyValue = function() {
            if(utils.isOnline()) {
                return null;
            } else {
                return "''";
            }
        };

        function genericQueryErrorHandler(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        }

        endpoints.fetchCurrentUserId = function() {
            var deferred = $q.defer();

            ds.getDataForCurrentObject("User", "Id").then(function(resp) {
                if(resp.success) {
                    if(utils.isOnline()) {
                        normalizeIdForOnline(resp.User);
                    }
                    deferred.resolve(resp.User.Id);
                } else {
                    (genericQueryErrorHandler(deferred))(resp.message);
                }
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchUserById = function(userId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.userById);
            queryConfig.where = queryConfig.where.replace(/_id/, '\'' + userId + '\'');
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchTerritoryByUser = function(userId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.territoryByUserId);
            queryConfig.where = queryConfig.where.replace(/_userId/, '\'' + userId + '\'');
            queryConfig.where = queryConfig.where.replace(/_true/, normalizeWhereClauseBooleanValue(true));
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAllTerritories = function() {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.territory);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAvailableObjects = function() {
            var deferred = $q.defer();
            ds.getAvailableObjects().then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchObjectMetadata = function(objectName) {
            var deferred = $q.defer();
            ds.getObjectMetadata({object: objectName}).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchVeevaMessages = function(msgList, locale) {
            var deferred = $q.defer();
            ds.getVeevaMessagesWithDefault(msgList, locale).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchFieldLabels = function(objectName, fieldNames) {
            var deferred = $q.defer();
            ds.getFieldLabels({object: objectName, fields: fieldNames}).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchObjectLabels = function(objectNames) {
            var deferred = $q.defer();
            ds.getObjectLabels(objectNames).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchPicklistLabels = function(objectName, fieldName) {
            var deferred = $q.defer();
            ds.getPicklistValueLabels(objectName, fieldName).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchRecordTypeLabels = function(objectName) {
            var deferred = $q.defer();
            ds.getRecordTypeLabels(objectName).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchSentMessageByUserId = function(userId, startDatetime, endDatetime) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.sentMessagesByUser);
            queryConfig.where = queryConfig.where.replace(/_userId/, '\'' + userId + '\'');
            queryConfig.where = queryConfig.where.replace(/_datetime1/, normalizeWhereClauseDateValue(startDatetime));
            queryConfig.where = queryConfig.where.replace(/_datetime2/, normalizeWhereClauseDateValue(endDatetime));
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAllSentMessageByUserId = function(userId) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.sentMessagesByUser);
            queryConfig.where = "OwnerId = '" + userId + "'";
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchMultichannelConsentByAccountIds = function(accountIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.multichannelConsentByAccountIds);
            queryConfig.where += ds.getInStatement(accountIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAccountByIds = function(accountIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.accountByIds);
            queryConfig.where += ds.getInStatement(accountIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAccountByTerritoryIds = function(territoryIds) {
            var deferred = $q.defer();

            // To get Account by TerritoryIds, we need to first get the territory related id from Group
            endpoints.fetchTerritoryGroupByRelatedId(territoryIds).then(function(groupResp) {
                // Then we can query account ids from AccountShare by the territory related id
                endpoints.fetchAccountShareByGroupIds(_.map(groupResp.Group, "Id")).then(function(accountShareResp) {
                    // Finally, query account object once we get the account ids from AccountShare
                    endpoints.fetchAccountByIds(_.map(accountShareResp.AccountShare, "AccountId")).then(function(accountResp) {
                        // Add territory data for each account
                        var accountWithTerritory = accountResp.Account.map(function(acct) {
                            // for each account, trace back to its accountShare record
                            var accountShareRecordsForAccountX = accountShareResp.AccountShare.filter(function(as) {
                                return as.AccountId === acct.Id;
                            });
                            // get the group ids found inside the accountShare records
                            var groupIdsForAccountX = _.map(accountShareRecordsForAccountX, "UserOrGroupId");
                            // filter the group records to only include the given group ids from above
                            var groupRecordsForAccountX = groupResp.Group.filter(function(g) {
                                return _.includes(groupIdsForAccountX, g.Id)
                            });
                            // get the territory ids from group records
                            acct._territoryIds = _.map(groupRecordsForAccountX, "RelatedId");
                            return acct;
                        });
                        deferred.resolve(accountWithTerritory);
                    }, genericQueryErrorHandler(deferred));
                }, genericQueryErrorHandler(deferred));
            }, genericQueryErrorHandler(deferred));

            return deferred.promise;
        };

        endpoints.fetchTerritoryGroupByRelatedId = function(relatedIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.territoryGroupByRelatedIds);
            queryConfig.where += ds.getInStatement(relatedIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchAccountShareByGroupIds = function(groupIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.accountShareByGroupIds);
            queryConfig.where += ds.getInStatement(groupIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchMultichannelContentByIds = function(mcContentIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.multichannelContentByIds);
            queryConfig.where += ds.getInStatement(mcContentIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchContentTypeByIds = function(contentTypeIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.contentTypeByIds);
            queryConfig.where += ds.getInStatement(contentTypeIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchWeChatRecordType = function() {
            var deferred = $q.defer();
            ds.queryRecord(queries.weChatRecordType).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        endpoints.fetchTargetTsfRecordByExternalIds = function(externalIds) {
            var deferred = $q.defer();
            var queryConfig = angular.copy(queries.tsfByExtId);
            queryConfig.where = queryConfig.where.replace(/_true/, normalizeWhereClauseBooleanValue(true));
            queryConfig.where += ds.getInStatement(externalIds);
            ds.queryRecord(queryConfig).then(deferred.resolve, genericQueryErrorHandler(deferred));
            return deferred.promise;
        };

        return endpoints;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("PermissionService", ["$q", "Endpoints", PermissionService])
    ;

    function PermissionService($q, endpoints) {
        var service = this;

        /**
         * Check if user have access to the given object list.
         * If objectList is omitted, check against the objects defined within Queries
         * @param queryObjects the map of query object
         * @returns {*}
         */
        service.checkObjectFLS = function(queryObjects) {
            var deferred = $q.defer();

            endpoints.fetchAvailableObjects().then(function(resp) {
                var availableObjects = resp.data;
                var unreachableObjects = {};

                for(var key in queryObjects) {
                    if(Object.prototype.hasOwnProperty.call(queryObjects, key)) {
                        var objName = queryObjects[key].object;
                        if(!availableObjects[objName]) {
                            unreachableObjects[objName] = [];
                        }
                    }
                }
                var unreachableObjectsList = _.keys(unreachableObjects);
                if(unreachableObjectsList.length > 0) {
                    console.error("Cannot read object: " + unreachableObjectsList.join(", "));
                }
                deferred.resolve(unreachableObjects);
            }, function(error) {
                deferred.reject(error);
            });

            return deferred.promise;
        };

        /**
         * Check if user have access to the given fields under the given object.
         * If both parameters are omitted, check against the objects and fields defined within Queries
         * @param queryObjects the map of query object
         * @returns {*}
         */
        service.checkFieldFLS = function(queryObjects) {
            var deferred = $q.defer();
            var checkFieldPromises = [];

            for(var key in queryObjects) {
                if(Object.prototype.hasOwnProperty.call(queryObjects, key)) {
                    var subDeferred = $q.defer();
                    checkFieldFlsHelper(queryObjects[key].object, queryObjects[key].fields, subDeferred);
                    checkFieldPromises.push(subDeferred.promise);
                }
            }

            $q.all(checkFieldPromises).then(function(comboResp) {
                var unreachableFieldsByObject = {};
                comboResp.forEach(function(resp) {
                    if(resp.fields && resp.object && !unreachableFieldsByObject[resp.object]) {
                        unreachableFieldsByObject[resp.object] = resp.fields;
                    }
                });
                deferred.resolve(unreachableFieldsByObject);
            }, function(error) {
                deferred.reject(error);
            });

            return deferred.promise;
        };

        /**
         * Check FLS for object and fields defined within Queries
         */
        service.checkObjectAndFieldFLS = function(queryObjects) {
            var deferred = $q.defer();

            service.checkObjectFLS(queryObjects).then(function(unreachableObjects) {
                var filteredQueryObjects = _.filter(queryObjects, function(qObj) {
                    return !angular.isDefined(unreachableObjects[qObj.object]);
                });
                service.checkFieldFLS(filteredQueryObjects).then(function(unreachableFieldsByObject) {
                    deferred.resolve(_.assign(unreachableObjects, unreachableFieldsByObject));
                }, function(error) {
                    deferred.reject(error);
                });
            }, function(error) {
                deferred.reject(error);
            });

            return deferred.promise;
        };

        function checkFieldFlsHelper(objectName, fieldList, deferred) {
            endpoints.fetchObjectMetadata(objectName).then(function(resp) {
                var availableFields = _.map(resp.data.fields, "name");
                var unknownFields = _.difference(fieldList, availableFields);
                var result = {};
                if(unknownFields.length > 0) {
                    console.error("Cannot read field: " + unknownFields.join(", ") + " from " + objectName);
                    result.object = objectName;
                    result.fields = unknownFields;
                }
                deferred.resolve(result);
            }, function(error) {
                deferred.reject(error);
            });
        }

        return service;
    }
}());

(function() {
    'use strict';

    angular.module('weChatApp')
        .factory('SentMessageService', ["$q", "Endpoints", SentMessageService])
    ;

    function SentMessageService($q, endpoints) {
        var service = this;

        service.getSentMessageByUserId = function(userId, filterDates, includeAccountName, includeMultichannelContent, includeContentType) {
            var deferred = $q.defer();

            if(!!filterDates) {
                endpoints.fetchSentMessageByUserId(userId, filterDates.startDatetime, filterDates.endDatetime).then(
                    sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType),
                    deferred.reject);
            } else {
                endpoints.fetchAllSentMessageByUserId(userId).then(
                    sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType),
                    deferred.reject);
            }

            return deferred.promise;
        };

        function sentMessageResponseProcessor(deferred, includeAccountName, includeMultichannelContent, includeContentType) {
            return function(resp) {
                var sentMessageResp = resp.Sent_Message_vod__c;
                var subQueries = [];

                if(includeAccountName) {
                    var accountIds = _.map(_.uniqBy(sentMessageResp, "Account_vod__c"), "Account_vod__c");
                    var subDeferred1 = $q.defer();
                    endpoints.fetchAccountByIds(accountIds).then(subDeferred1.resolve, subDeferred1.reject);
                    subQueries.push({
                        name: "Account",
                        promise: subDeferred1.promise
                    });
                }

                if(includeMultichannelContent) {
                    var contentIds = _.map(_.uniqBy(sentMessageResp, "Content_vod__c"), "Content_vod__c");
                    var subDeferred2 = $q.defer();
                    endpoints.fetchMultichannelContentByIds(contentIds).then(subDeferred2.resolve, subDeferred2.reject);
                    subQueries.push({
                        name: "Multichannel_Content_vod__c",
                        promise: subDeferred2.promise
                    });
                }

                if(includeContentType) {
                    var contentTypeIds = _.map(_.uniqBy(sentMessageResp, "Content_Type_vod__c"), "Content_Type_vod__c");
                    var subDeferred3 = $q.defer();
                    endpoints.fetchContentTypeByIds(contentTypeIds).then(subDeferred3.resolve, subDeferred3.reject);
                    subQueries.push({
                        name: "Content_Type_vod__c",
                        promise: subDeferred3.promise
                    });
                }

                if(subQueries.length === 0) {
                    deferred.resolve(sentMessageResp);
                } else {
                    $q.all(_.map(subQueries, "promise")).then(function(subQueriesResp) {
                        subQueries.forEach(function(q, i) {
                            var subQueryType = q.name;
                            var accountIdToNameMap, mcContentIdToNameMap, contentTypeIdToNameMap;

                            if(subQueryType === "Account") {
                                accountIdToNameMap = generateIdToNameMap(subQueriesResp[i].Account);
                            } else if(subQueryType === "Multichannel_Content_vod__c") {
                                mcContentIdToNameMap = generateIdToNameMap(subQueriesResp[i].Multichannel_Content_vod__c);
                            } else if(subQueryType === "Content_Type_vod__c") {
                                contentTypeIdToNameMap = generateIdToNameMap(subQueriesResp[i].Content_Type_vod__c);
                            }

                            sentMessageResp.forEach(function(sm) {
                                if(!!sm.Account_vod__c && !!accountIdToNameMap) {
                                    sm.accountName = accountIdToNameMap[sm.Account_vod__c] || '';
                                }
                                if(!!sm.Content_vod__c && !!mcContentIdToNameMap) {
                                    sm.contentName = mcContentIdToNameMap[sm.Content_vod__c] || '';
                                }
                                if(!!sm.Content_Type_vod__c && !!contentTypeIdToNameMap) {
                                    sm.serviceAccountName = contentTypeIdToNameMap[sm.Content_Type_vod__c] || '';
                                }
                            });
                        });
                        deferred.resolve(sentMessageResp);
                    }, function(error) {
                        console.warn(error);
                        deferred.resolve(sentMessageResp);
                    });
                }
            };
        }

        function generateIdToNameMap(objectList) {
            var map = {};
            objectList.forEach(function(obj) {
                map[obj.Id] = obj.Name;
            });
            return map;
        }

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("TerritoryService", ["$q", "Endpoints", TerritoryService])
    ;

    function TerritoryService($q, endpoints) {
        var service = this;
        var territoryHierarchy = {};
        var userTerritoriesMap = {};

        service.getAccountsUnderUserTerritory = function(userId) {
            var deferred = $q.defer();
            var subQueries = [{
                name: "Territory",
                promise: getAllTerritories()
            }, {
                name: "UserTerritory",
                promise: getUserTerritories(userId)
            }];

            $q.all(_.map(subQueries, "promise")).then(function(subQueriesResp) {
                var allTerritories = subQueriesResp[0].Territory;
                var userTerritories = subQueriesResp[1].UserTerritory;
                userTerritoriesMap = getQualifiedUserTerritories(allTerritories, userTerritories);

                endpoints.fetchAccountByTerritoryIds(_.map(userTerritoriesMap, "Id")).then(function(accountResp) {
                    accountResp = accountResp.map(function(acct) {
                        acct.territories = [];
                        acct._territoryIds.forEach(function(tId) {
                            acct.territories.push(userTerritoriesMap[tId]);
                        });
                        delete acct._territoryIds;
                        return acct;
                    });
                    deferred.resolve(accountResp);
                }, deferred.reject);
            }, deferred.reject);

            return deferred.promise;
        };

        service.getTerritoriesMapForCurrentUser = function() {
            return userTerritoriesMap;
        };

        function getAllTerritories() {
            var deferred = $q.defer();
            endpoints.fetchAllTerritories().then(deferred.resolve, deferred.reject);
            return deferred.promise;
        }

        function getUserTerritories(userId) {
            var deferred = $q.defer();
            endpoints.fetchTerritoryByUser(userId).then(deferred.resolve, deferred.reject);
            return deferred.promise;
        }

        function getQualifiedUserTerritories(allTerritories, userTerritories) {
            var territoryMap = {};

            // Initial pass to group territories by their parent
            allTerritories.forEach(function(t) {
                var parentId = t.ParentTerritoryId || "root";
                if(!territoryHierarchy[parentId]) {
                    territoryHierarchy[parentId] = [];
                }
                territoryHierarchy[parentId].push(t);
                territoryMap[t.Id] = t;
            });

            var greedyChildIds = [];
            userTerritories.forEach(function(ut) {
                var childIds = findChildTerritories(ut.TerritoryId);
                greedyChildIds = greedyChildIds.concat(childIds);
            });

            var qualifiedUserTerritories = {};
            greedyChildIds.forEach(function(childId) {
                qualifiedUserTerritories[childId] = territoryMap[childId];
            });

            return qualifiedUserTerritories;
        }

        function findChildTerritories(baseTerritoryId) {
            if(!!territoryHierarchy[baseTerritoryId]) {
                var childIds = [baseTerritoryId];
                territoryHierarchy[baseTerritoryId].forEach(function(t) {
                    childIds = childIds.concat(findChildTerritories(t.Id));
                });
                return childIds;
            } else {
                return [baseTerritoryId];
            }
        }

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("TranslationService", ["$q", "VeevaMessages", "Endpoints", TranslationService])
    ;

    function TranslationService($q, veevaMessages, endpoints) {
        var service = this;
        var translatedMessages;
        var translatedObjectLabels;
        var translatedFieldLabels;
        var translatedPicklistLabels;
        var translatedRecordTypeLabels;

        service.getFieldLabels = function(fieldLabelMap) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            for(var key in fieldLabelMap) {
                if(Object.prototype.hasOwnProperty.call(fieldLabelMap, key)) {
                    var subDeferred = $q.defer();

                    translationQueue.push({
                        object: key,
                        fields: fieldLabelMap[key],
                        deferred: subDeferred
                    });

                    translationPromises.push(subDeferred.promise);
                }
            }

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchFieldLabels(transBlock.object, transBlock.fields).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(resp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    transBlock.fields.forEach(function(transField, j) {
                        var objectName = transBlock.object;
                        var fieldName = transField;
                        if(!translationResult[objectName]) {
                            translationResult[objectName] = {};
                        }
                        translationResult[objectName][fieldName] = resp[i][j].display;
                    });
                });
                translatedFieldLabels = translationResult;
                deferred.resolve(translatedFieldLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getObjectLabels = function(fieldLabelMap) {
            var deferred = $q.defer();
            var objectList = [];

            for(var key in fieldLabelMap) {
                if(Object.prototype.hasOwnProperty.call(fieldLabelMap, key)) {
                    objectList.push(key);
                }
            }

            endpoints.fetchObjectLabels(objectList).then(function(resp) {
                translatedObjectLabels = setupTranslatedObjectLabels(resp);
                deferred.resolve(translatedObjectLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getPicklistLabels = function(picklistLabelMap) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            for(var key in picklistLabelMap) {
                if(Object.prototype.hasOwnProperty.call(picklistLabelMap, key)) {
                    var picklistFields = picklistLabelMap[key];
                    picklistFields.forEach(function(pField) {
                        var subDeferred = $q.defer();
                        translationQueue.push({
                            object: key,
                            field: pField,
                            deferred: subDeferred
                        });
                        translationPromises.push(subDeferred.promise);
                    });
                }
            }

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchPicklistLabels(transBlock.object, transBlock.field).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(combinedResp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    var objectName = transBlock.object;
                    var fieldName = transBlock.field;
                    if(!translationResult[objectName]) {
                        translationResult[objectName] = {};
                    }
                    translationResult[objectName][fieldName] = combinedResp[i][objectName][fieldName];
                });
                translatedPicklistLabels = translationResult;
                deferred.resolve(translatedPicklistLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getRecordTypeLabels = function(recordTypeLabelList) {
            var deferred = $q.defer();
            var translationQueue = [];
            var translationPromises = [];

            recordTypeLabelList.forEach(function(objectName) {
                var subDeferred = $q.defer();
                translationQueue.push({
                    object: objectName,
                    deferred: subDeferred
                });
                translationPromises.push(subDeferred.promise);
            });

            translationQueue.forEach(function(transBlock) {
                endpoints.fetchRecordTypeLabels(transBlock.object).then(function(resp) {
                    transBlock.deferred.resolve(resp);
                }, rejectionHandler(transBlock.deferred));
            });

            $q.all(translationPromises).then(function(combinedResp) {
                var translationResult = {};
                translationQueue.forEach(function(transBlock, i) {
                    var objectName = transBlock.object;
                    translationResult[objectName] = combinedResp[i][objectName];
                });
                translatedRecordTypeLabels = translationResult;
                deferred.resolve(translatedRecordTypeLabels);
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        service.getVeevaMessages = function(locale, defaultLocale) {
            var deferred = $q.defer();
            var messageTokens = _.values(veevaMessages);

            endpoints.fetchVeevaMessages(messageTokens, locale).then(function(resp) {
                if(_.values(resp).length === 0) {
                    endpoints.fetchVeevaMessages(messageTokens, defaultLocale).then(function(resp) {
                        translatedMessages = setupTranslatedMessages(resp);
                        deferred.resolve(translatedMessages);
                    });
                } else {
                    translatedMessages = setupTranslatedMessages(resp);
                    deferred.resolve(translatedMessages);
                }
            }, rejectionHandler(deferred));

            return deferred.promise;
        };

        /**
         * Method used by message filter to show translated message on the UI
         * Accepted parameters include:
         * 1. message key as string
         *      e.g. 'all'
         * 2. an Array contains the message key as the first element, follow by any replaceable values
         *      e.g. ['progress', '50', '100']
         *      note: the message for 'progress' could be: "{0} out of {1} completed"
         * @returns {*}
         */
        service.getMessage = function() {
            var key = arguments[0];
            if(translatedMessages[key]) {
                var message = translatedMessages[key];
                for(var i = 1; i < arguments.length; i++) {
                    var token = arguments[i];
                    var marker = i - 1;
                    message = message.replace('{' + marker + '}', token);
                }
                return message;
            } else {
                return key;
            }
        };

        function setupTranslatedMessages(veevaMessageResp) {
            var translatedMsg = {};

            angular.forEach(veevaMessages, function(msgToken, key) {
                var foundTranslation = _.find(veevaMessageResp, function(rawMsg) {
                    return msgToken.msgName === rawMsg.Name.value && msgToken.msgCategory === rawMsg.Category_vod__c.value;
                });

                if(foundTranslation) {
                    translatedMsg[key] = foundTranslation.Text_vod__c.display;
                }
            });

            return translatedMsg;
        }

        function setupTranslatedObjectLabels(objLabelResp) {
            var translatedObjLabels = {};

            angular.forEach(objLabelResp, function(msgToken, key) {
                if(typeof msgToken === 'object' && msgToken.length > 0) {
                    translatedObjLabels[key] = msgToken[0];
                }
            });

            return translatedObjLabels;
        }

        function rejectionHandler(deferred) {
            return function(error) {
                console.error(error);
                deferred.reject(error);
            };
        }

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("TsfService", ["$q", "Endpoints", TsfService])
    ;

    function TsfService($q, endpoints) {
        var service = this;

        service.getTargetAccountByExternalIds = function(externalIds) {
            var deferred = $q.defer();
            endpoints.fetchTargetTsfRecordByExternalIds(externalIds).then(function(resp) {
                deferred.resolve(resp.TSF_vod__c);
            });
            return deferred.promise;
        };

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("UserService", ["$q", "Endpoints", UserService])
    ;

    function UserService($q, endpoints) {
        var service = this;

        service.getCurrentUser = function() {
            var deferred = $q.defer();

            endpoints.fetchCurrentUserId().then(function(userId) {
                endpoints.fetchUserById(userId).then(function(resp) {
                    deferred.resolve(resp.User[0]);
                });
            });

            return deferred.promise;
        };

        return service;
    }
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .factory("Utility", [Utility])
    ;

    function Utility() {
        var utility = this;

        /**
         * Helper function to calculate the percentage of two given number
         * @param dividend
         * @param divisor
         * @param decimal The number of decimal point the result should keep
         * @returns {*} A string representing the percentage number
         */
        utility.calculatePercentage = function(dividend, divisor, decimal) {
            if(divisor === 0) {
                return 0;
            }
            return ((dividend/divisor)*100).toFixed(decimal);
        };

        utility.genericQueryErrorHandler = function(deferred) {
            return function(error) {
                deferred.reject(error);
            };
        };

        utility.parseStringToDateValue = function(string, format) {
            var momentDate;
            if(format) {
                momentDate = moment(string, format);
            } else {
                momentDate = moment(string);
            }
            return momentDate.isValid() ? momentDate.valueOf() : null;
        };

        return utility;
    }
})();

(function() {
    "use strict";

    angular.module("weChatApp")
        .value("AppConstants", {
            FULL_DATE_TIME_FORMAT: "YYYY-MM-DDTHH:mm:ss.SSS[Z]",
            YEAR_MONTH_DAY_FORMAT: "YYYY-MM-DD",
            YEAR_MONTH_FORMAT: "YYYY-MM",
            SENT_MESSAGE: "SENT_MESSAGE",
            CONSENT: "CONSENT",
            UPDATE_DOUGHNUT_CHART_EVENT: "UPDATE_DOUGHNUT_CHART_EVENT"
        });
}());

(function() {
    "use strict";

    /**
     * Defined the object and field name here so that they will be pick up by translation service later on
     * To translate object label only, add the object api name as key and a empty array as value:
     * object_api_name: []
     *
     * To translate field label, add the object api name as key and the field(s) as value:
     * object_api_name: [field1, field2]
     */
    angular.module("weChatApp")
        .value("FieldLabels", {
            Sent_Message_vod__c: ["Content_vod__c", "Account_vod__c", "Sent_Datetime_vod__c", "Last_Open_Datetime_vod__c", "Total_Duration_vod__c"],
            Multichannel_Consent_vod__c: ["Account_vod__c", "Opt_Type_vod__c", "Capture_Datetime_vod__c", "Opt_Expiration_Date_vod__c"]
        });
}());
(function() {
    "use strict";

    /**
     * Usage - Place the picklist field here as key value pair so that it can be picked up
     * by as the application startup via translation service
     * e.g.
     * ObjectApiName: ["field1","field2"]
     */
    angular.module("weChatApp")
        .value("PicklistLabels", {
            Multichannel_Consent_vod__c: ["Opt_Type_vod__c"]
        });
}());
(function() {
    "use strict";

    /**
     * Queries holds all the query object entity used within mdApp.
     * Queries should only used by Endpoints service to centralize the control, with the exception to PermissionService
     * used for auto checking FLS on the object and fields defined here.
     */
    angular.module("weChatApp")
        .value("Queries", {
            userById: {
                object: "User",
                fields: ["Id", "Name", "LanguageLocaleKey", "LocaleSidKey", "TimeZoneSidKey"],
                where: "Id = _id"
            },
            territoryByUserId: {
                object: "UserTerritory",
                fields: ["TerritoryId"],
                where: "UserId = _userId AND IsActive = _true"
            },
            territory: {
                object: "Territory",
                fields: ["Id", "ParentTerritoryId", "Name"]
            },
            territoryGroupByRelatedIds: {
                object: 'Group',
                fields: ['Id', 'RelatedId'],
                where: 'Type = \'Territory\' AND RelatedId IN '
            },
            accountShareByGroupIds: {
                object: 'AccountShare',
                fields: ['Id', 'AccountId', 'UserOrGroupId'],
                where: 'UserOrGroupId IN '
            },
            sentMessagesByUser: {
                object: "Sent_Message_vod__c",
                fields: ["Id", "OwnerId", "Account_vod__c", "Content_vod__c", "Sent_Datetime_vod__c", "Opened_vod__c",
                    "Last_Open_Datetime_vod__c", "Content_Type_vod__c", "Total_Duration_vod__c"],
                where: "OwnerId = _userId AND Sent_Datetime_vod__c >= _datetime1 AND Sent_Datetime_vod__c <= _datetime2"
            },
            multichannelConsentByAccountIds: {
                object: "Multichannel_Consent_vod__c",
                fields: ["Id", "Account_vod__c", "Opt_Type_vod__c", "Capture_Datetime_vod__c", "Opt_Expiration_Date_vod__c",
                    "RecordTypeId", "Content_Type_vod__c"],
                where: "(Opt_Type_vod__c = 'Opt_In_vod' OR Opt_Type_vod__c = 'Opt_In_Pending_vod') AND Account_vod__c IN "
            },
            accountByIds: {
                object: "Account",
                fields: ["Id", "Name"],
                where: "Id IN "
            },
            multichannelContentByIds: {
                object: "Multichannel_Content_vod__c",
                fields: ["Id", "Name"],
                where: "Id IN "
            },
            contentTypeByIds: {
                object: "Content_Type_vod__c",
                fields: ["Id", "Name"],
                where: "Id IN "
            },
            weChatRecordType: {
                object: 'RecordType',
                fields: ["Id", "DeveloperName", "SobjectType"],
                where: "SobjectType = \'Multichannel_Consent_vod__c\' AND DeveloperName = \'WeChat_vod\'"
            },
            tsfByExtId: {
                object: "TSF_vod__c",
                fields: ["Account_vod__c", "Territory_vod__c", "External_Id_vod__c", "My_Target_vod__c"],
                where: "My_Target_vod__c = _true AND External_Id_vod__c IN "
            }
        })
    ;
}());

(function() {
    "use strict";

    angular.module("weChatApp")
        .value("RecordTypeLabels", [
        ]);
}());
(function() {
    "use strict";

    angular.module("weChatApp")
        .value("VeevaMessages", {
            ACTIVITY: {msgName: 'ACTIVITY', msgCategory: 'FieldReporting'},
            VIEW_ALL: {msgName: 'View_All', msgCategory: 'Common'},
            LAST_DAYS: {msgName: 'LAST_DAYS', msgCategory: 'FieldReporting'},
            BACK: {msgName: "BACK", msgCategory: "Common"},
            SERVICE_ACCOUNT: {msgName: "SERVICE_ACCOUNT", msgCategory: "FieldReporting"},
            OPENED: {msgName: "OPENED", msgCategory: "FieldReporting"},
            UNOPENED: {msgName: "UNOPENED", msgCategory: "FieldReporting"},
            VIEW_MESSAGES: {msgName: "VIEW_MESSAGES", msgCategory: "FieldReporting"},
            OPEN_RATE: {msgName: "OPEN_RATE", msgCategory: "FieldReporting"},
            OPEN_RATE_DESCRIPTION: {msgName: "OPEN_RATE_DESCRIPTION", msgCategory: "FieldReporting"},
            COVERAGE: {msgName: "COVERAGE", msgCategory: "FieldReporting"},
            VIEW_CONSENT_STATUS: {msgName: "VIEW_CONSENT_STATUS", msgCategory: "FieldReporting"},
            TARGET_HCP_OPT_IN_RATE: {msgName: "TARGET_HCP_OPT_IN_RATE", msgCategory: "FieldReporting"},
            TARGET_HCP_OPT_IN_RATE_DESCRIPTION: {msgName: "TARGET_HCP_OPT_IN_RATE_DESCRIPTION", msgCategory: "FieldReporting"},
            TARGET_HCP_SENT_MESSAGE_COVERAGE: {msgName: "TARGET_HCP_SENT_MESSAGE_COVERAGE", msgCategory: "FieldReporting"},
            TARGET_HCP_SENT_MESSAGE_COVERAGE_DESCRIPTION: {msgName: "TARGET_HCP_SENT_MESSAGE_COVERAGE_DESCRIPTION", msgCategory: "FieldReporting"}
        })
    ;
}());

angular.module('weChatApp').run(['$templateCache', function($templateCache) {
  'use strict';

  $templateCache.put('views/custom-table.html',
    "<div class=\"custom-table\">\n" +
    "    <div class=\"custom-table-row header-row\">\n" +
    "        <div class=\"custom-table-cell\" ng-repeat=\"header in headers\" ng-click=\"sortColumn(header)\">{{header.label}}</div>\n" +
    "    </div>\n" +
    "    <div class=\"custom-table-row body-row\" ng-class=\"{'odd-row': $odd}\" ng-repeat=\"row in rows | orderBy:orderKey\">\n" +
    "        <div class=\"custom-table-cell\" ng-repeat=\"header in headers\">{{row[header.name] | DateFormatter:header.isDate}}</div>\n" +
    "    </div>\n" +
    "</div>"
  );


  $templateCache.put('views/dashboard-tile.html',
    "<div class=\"dashboard-tile\" ng-if=\"show\">\n" +
    "    <div class=\"dashboard-tile-header\" ng-if=\"mainTitle\">\n" +
    "        <span class=\"main-title\">{{mainTitle}}</span>\n" +
    "        <span class=\"sub-title\" ng-if=\"subTitle\">({{subTitle}})</span>\n" +
    "        <span class=\"go-right\">\n" +
    "            <span class=\"action-section\" ng-if=\"actionLabel\" ng-click=\"actionCallback()\">\n" +
    "                <span>{{actionLabel}}</span>\n" +
    "                <span class=\"fa fa-angle-right icon\"></span>\n" +
    "            </span>\n" +
    "        </span>\n" +
    "    </div>\n" +
    "    <div class=\"dashboard-tile-body\">\n" +
    "        <ng-transclude></ng-transclude>\n" +
    "    </div>\n" +
    "</div>"
  );


  $templateCache.put('views/info-card.html',
    "<div class=\"info-card-container\">\n" +
    "    <div class=\"info-card-page-title\" ng-if=\"!!mainTitle\">\n" +
    "        <span>{{mainTitle}}</span>\n" +
    "        <span ng-if=\"!!subTitle\">({{subTitle}})</span>\n" +
    "    </div>\n" +
    "    <div ng-repeat=\"card in cardBody\" class=\"info-card\">\n" +
    "        <div class=\"info-card-header\">{{card.cardHeader}}</div>\n" +
    "        <div class=\"info-card-body\">\n" +
    "            <div ng-repeat=\"header in cardHeaders\" class=\"info-card-row\">\n" +
    "                <div class=\"head-col\">{{header.label}}</div>\n" +
    "                <div>{{card[header.name] | DateFormatter:header.isDate}}</div>\n" +
    "            </div>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "</div>"
  );


  $templateCache.put('views/wechat-rep-dashboard.html',
    "<div class=\"dashboard\">\n" +
    "    <!-- global filter section -->\n" +
    "    <div class=\"header-section\" ng-class=\"{'invisible':!dash.sentMessageReady && !dash.consentReady}\">\n" +
    "        <div class=\"header-section-item align-right\" ng-if=\"!dash.detailMode\">\n" +
    "            <select class=\"no-default-appearance\"\n" +
    "                    ng-options=\"filterOption.label for filterOption in dash.globalDateFilterOptions\"\n" +
    "                    ng-model=\"dash.selectedDateFilter\" ng-change=\"dash.globalFilterOnChange()\">\n" +
    "            </select>\n" +
    "            <span class=\"fa fa-angle-down icon\"></span>\n" +
    "        </div>\n" +
    "        <div class=\"header-section-item link go-left\" ng-if=\"dash.detailMode\">\n" +
    "            <span ng-click=\"dash.toggleDetailMode()\">\n" +
    "                <span class=\"fa fa-angle-left icon\"></span>\n" +
    "                <span>{{'BACK' | TranslationFilter}}</span>\n" +
    "            </span>\n" +
    "        </div>\n" +
    "        <div class=\"header-section-item go-right\" ng-if=\"dash.detailMode && dash.showSentMessageDetail\">\n" +
    "            <span>{{dash.selectedDateFilter.label}}</span>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "\n" +
    "    <div class=\"body-section overview\" ng-if=\"!dash.detailMode\">\n" +
    "\n" +
    "        <!-- Sent Message section -->\n" +
    "        <dashboard-tile main-title=\"{{'ACTIVITY' | TranslationFilter}}\"\n" +
    "                        action-label=\"{{'VIEW_MESSAGES' | TranslationFilter}}\"\n" +
    "                        action-callback=\"dash.viewAllSentMessages()\"\n" +
    "                        show=\"dash.sentMessageReady\">\n" +
    "            <div class=\"dashboard-tile-body-item\">\n" +
    "                <!-- iPad -->\n" +
    "                <div class=\"sent-message-section ipad-only\">\n" +
    "                    <div class=\"sm-percentage\">{{dash.messageOpenRate | PercentageFormatter}}</div>\n" +
    "                    <div class=\"sm-description\">\n" +
    "                        <div>{{'OPEN_RATE' | TranslationFilter}}</div>\n" +
    "                        <div>{{'OPEN_RATE_DESCRIPTION' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                </div>\n" +
    "                <!-- iPhone -->\n" +
    "                <div class=\"sent-message-section iphone-only\">\n" +
    "                    <div class=\"sm-description\">\n" +
    "                        <div>{{'OPEN_RATE' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                    <div class=\"sm-percentage\">{{dash.messageOpenRate | PercentageFormatter}}</div>\n" +
    "                </div>\n" +
    "            </div>\n" +
    "        </dashboard-tile>\n" +
    "\n" +
    "        <!-- Territory Coverage section -->\n" +
    "        <dashboard-tile main-title=\"{{'COVERAGE' | TranslationFilter}}\"\n" +
    "                        action-label=\"{{'VIEW_CONSENT_STATUS' | TranslationFilter}}\"\n" +
    "                        action-callback=\"dash.viewAllConsents()\"\n" +
    "                        show=\"dash.consentReady\">\n" +
    "            <!-- iPad -->\n" +
    "            <div class=\"dashboard-tile-body-item ipad-only\">\n" +
    "                <div class=\"territory-coverage-section\">\n" +
    "                    <div class=\"tc-percentage\">\n" +
    "                        <doughnut-chart name=\"OPT_IN_RATE\" width=\"140px\" color1=\"rgb(56,205,184)\" color2=\"rgb(0,122,127)\"\n" +
    "                                        percentage=\"dash.hcpOptInPercentage\">\n" +
    "                        </doughnut-chart>\n" +
    "                    </div>\n" +
    "                    <div class=\"tc-description\">\n" +
    "                        <div>{{'TARGET_HCP_OPT_IN_RATE' | TranslationFilter}}</div>\n" +
    "                        <div>{{'TARGET_HCP_OPT_IN_RATE_DESCRIPTION' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                </div>\n" +
    "                <div class=\"territory-coverage-section\">\n" +
    "                    <div class=\"tc-percentage\">\n" +
    "                        <doughnut-chart name=\"SENT_MESSAGE_COVERAGE\" width=\"140px\" color1=\"rgb(56,205,184)\" color2=\"rgb(0,122,127)\"\n" +
    "                                        percentage=\"dash.hcpSentMessageCoverPercentage\">\n" +
    "                        </doughnut-chart>\n" +
    "                    </div>\n" +
    "                    <div class=\"tc-description\">\n" +
    "                        <div>{{'TARGET_HCP_SENT_MESSAGE_COVERAGE' | TranslationFilter}}</div>\n" +
    "                        <div>{{'TARGET_HCP_SENT_MESSAGE_COVERAGE_DESCRIPTION' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                </div>\n" +
    "            </div>\n" +
    "\n" +
    "            <!-- iPhone -->\n" +
    "            <div class=\"dashboard-tile-body-item iphone-only\">\n" +
    "                <div class=\"territory-coverage-section extra-padding-bottom\">\n" +
    "                    <div class=\"tc-description\">\n" +
    "                        <div>{{'TARGET_HCP_OPT_IN_RATE' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                    <div class=\"tc-percentage\">\n" +
    "                        <doughnut-chart name=\"OPT_IN_RATE\" width=\"140px\" color1=\"rgb(56,205,184)\" color2=\"rgb(0,122,127)\"\n" +
    "                                        percentage=\"dash.hcpOptInPercentage\">\n" +
    "                        </doughnut-chart>\n" +
    "                    </div>\n" +
    "                </div>\n" +
    "                <div class=\"territory-coverage-section\">\n" +
    "                    <div class=\"tc-description\">\n" +
    "                        <div>{{'TARGET_HCP_SENT_MESSAGE_COVERAGE' | TranslationFilter}}</div>\n" +
    "                    </div>\n" +
    "                    <div class=\"tc-percentage\">\n" +
    "                        <doughnut-chart name=\"SENT_MESSAGE_COVERAGE\" width=\"140px\" color1=\"rgb(56,205,184)\" color2=\"rgb(0,122,127)\"\n" +
    "                                        percentage=\"dash.hcpSentMessageCoverPercentage\">\n" +
    "                        </doughnut-chart>\n" +
    "                    </div>\n" +
    "                </div>\n" +
    "            </div>\n" +
    "        </dashboard-tile>\n" +
    "    </div>\n" +
    "\n" +
    "    <div class=\"body-section detail-view\" ng-if=\"dash.detailMode\">\n" +
    "\n" +
    "        <div ng-if=\"dash.showSentMessageDetail\">\n" +
    "            <!-- iPad -->\n" +
    "            <dashboard-tile main-title=\"{{dash.sentMessageObjectLabel}}\"\n" +
    "                            show=\"true\" class=\"ipad-only\">\n" +
    "                <custom-table headers=\"dash.sentMessageTableHeaders\"\n" +
    "                              rows=\"dash.sentMessageTableBody\">\n" +
    "                </custom-table>\n" +
    "            </dashboard-tile>\n" +
    "            <!-- iPhone -->\n" +
    "            <info-card card-headers=\"dash.sentMessageTableHeaders\"\n" +
    "                       card-body=\"dash.sentMessageInfoCard\"\n" +
    "                       class=\"iphone-only\">\n" +
    "            </info-card>\n" +
    "        </div>\n" +
    "\n" +
    "        <div ng-if=\"dash.showConsentDetail\">\n" +
    "            <!-- iPad -->\n" +
    "            <dashboard-tile main-title=\"{{dash.consentObjectLabel}}\"\n" +
    "                            show=\"true\" class=\"ipad-only\">\n" +
    "                <custom-table headers=\"dash.consentTableHeaders\"\n" +
    "                              rows=\"dash.consentTableBody\">\n" +
    "                </custom-table>\n" +
    "            </dashboard-tile>\n" +
    "            <!-- iPhone -->\n" +
    "            <info-card card-headers=\"dash.consentTableHeaders\"\n" +
    "                       card-body=\"dash.consentInfoCard\"\n" +
    "                       class=\"iphone-only\">\n" +
    "            </info-card>\n" +
    "        </div>\n" +
    "    </div>\n" +
    "</div>"
  );

}]);
